==> sciagraph-2022.6.2-cp38-cp38-manylinux2014_x86_64.whl: sciagraph/api.py <==
"""Public API for interacting with Sciagraph."""

from datetime import datetime, timezone
from typing import Optional
import logging
from dataclasses import dataclass, asdict
from pathlib import Path
from ctypes import CDLL, c_uint32, c_char, POINTER

__all__ = ["ReportResult", "set_job_id"]

_LOGGER = logging.getLogger("sciagraph")

_DOWNLOAD_INSTRUCTIONS = """\
Successfully uploaded the Sciagraph profiling report.

Job start time: {job_time}
Job ID: {job_id}

{local_storage}\
An encrypted copy of the report was uploaded to the Sciagraph storage server.
To download the report, run the following on Linux/Windows/macOS, Python 3.7+.

If you're inside a virtualenv:

    pip install --upgrade sciagraph-report

Otherwise:

    pip install --user --upgrade sciagraph-report

Then:

    python -m sciagraph_report download {download_key} {decryption_key}

If you have trouble installing sciagraph-report, please read the documentation:

https://sciagraph.com/docs/howto/report-viewer
"""

_STORAGE_INSTRUCTIONS = """\
Successfully stored the Sciagraph profiling report.

Job start time: {job_time}
Job ID: {job_id}

The report was stored in {report_path}{addendum}
"""

_TRIAL_MODE_ADDENDUM = """

WARNING: You are running in trial mode, which limits you to profiling only the
first 60 seconds of a job. To profile jobs of unlimited length, read the docs
on setting up Sciagraph for production:

https://www.sciagraph.com/docs/howto/setup
"""


@dataclass
class ReportResult:
    """
    Information about how to download uploaded profiling report.

    This will get logged by Sciagraph when profiling is finished.
    """

    job_time: str
    job_id: str
    download_key: Optional[str]
    decryption_key: Optional[str]
    report_path: Optional[Path]
    # Private attribute for now, don't rely on it:
    _trial_mode: bool

    def __str__(self):
        if self.download_key is not None and self.decryption_key is not None:
            if self.report_path is not None:
                local_storage = (
                    f"The report was stored locally at path {self.report_path}\n\n"
                )
            else:
                local_storage = ""
            return _DOWNLOAD_INSTRUCTIONS.format(
                **asdict(self), local_storage=local_storage
            )
        else:
            data = asdict(self)
            if self._trial_mode:
                data["addendum"] = _TRIAL_MODE_ADDENDUM
            else:
                data["addendum"] = ""
            return _STORAGE_INSTRUCTIONS.format(**data)


_UNKNOWN_JOB_ID = "Unknown, see docs to learn how to set this"


def _log_result(
    job_secs_since_epoch: int,
    job_id: Optional[str],
    download_key: Optional[str],
    decryption_key: Optional[str],
    report_path: Optional[Path],
    trial_mode: bool,
):
    """Log a ``ReportResult``."""
    if job_id is None:
        job_id = _UNKNOWN_JOB_ID
    job_time = datetime.fromtimestamp(job_secs_since_epoch, timezone.utc).isoformat()
    report = ReportResult(
        job_time=job_time,
        job_id=job_id,
        download_key=download_key,
        decryption_key=decryption_key,
        report_path=report_path,
        _trial_mode=trial_mode,
    )
    _LOGGER.warning(report)


def set_job_id(job_id: str):
    """
    Set the current job's ID; it will then be included in the resulting report
    and logged message.
    """
    if not isinstance(job_id, str):
        job_id = f"You called set_job_id() wih a non-string value: {repr(job_id)}"

    # The symbols in the current running process:
    dll = CDLL(None)
    f = getattr(dll, "sciagraph_set_job_id", None)
    if f is None:
        # Profiling is not actually happening, do nothing
        return

    f.argtypes = [POINTER(c_char), c_uint32]
    f.restype = None
    job_id_bytes = job_id.encode("utf-8")
    f(job_id_bytes, len(job_id_bytes))

==> sciagraph-2022.7.2-cp38-cp38-manylinux2014_x86_64.whl: sciagraph/api.py <==
"""Public API for interacting with Sciagraph."""

from __future__ import annotations

from datetime import datetime, timezone
from typing import Optional, Union
import logging
from dataclasses import dataclass, asdict
from os import PathLike, makedirs
from pathlib import Path
from ctypes import PyDLL, py_object
from contextlib import contextmanager
import sys

__all__ = ["ReportResult", "set_job_id"]

_LOGGER = logging.getLogger("sciagraph")

_DOWNLOAD_INSTRUCTIONS = """\
Successfully uploaded the Sciagraph profiling report.

Job start time: {job_time}
Job ID: {job_id}

{local_storage}\
An encrypted copy of the report was uploaded to the Sciagraph storage server.
To download the report, run the following on Linux/Windows/macOS, Python 3.7+.

If you're inside a virtualenv:

    pip install --upgrade sciagraph-report

Otherwise:

    pip install --user --upgrade sciagraph-report

Then:

    python -m sciagraph_report download {download_key} {decryption_key}

If you have trouble installing sciagraph-report, please read the documentation:

https://sciagraph.com/docs/howto/report-viewer
"""

_STORAGE_INSTRUCTIONS = """\
Successfully stored the Sciagraph profiling report.

Job start time: {job_time}
Job ID: {job_id}

The report was stored in {report_path}{addendum}
"""

_TRIAL_MODE_ADDENDUM = """

WARNING: You are running in trial mode, which limits you to profiling only the
first 60 seconds of a job. To profile jobs of unlimited length, read the docs
on setting up Sciagraph for production:

https://www.sciagraph.com/docs/howto/setup
"""


@dataclass
class ReportResult:
    """
    Information about how to download uploaded profiling report.

    This will get logged by Sciagraph when profiling is finished.
    """

    job_time: str
    job_id: str
    download_key: Optional[str]
    decryption_key: Optional[str]
    report_path: Optional[Path]
    # Private attribute for now, don't rely on it:
    _trial_mode: bool

    def __str__(self):
        if self.download_key is not None and self.decryption_key is not None:
            if self.report_path is not None:
                local_storage = (
                    f"The report was stored locally at path {self.report_path}\n\n"
                )
            else:
                local_storage = ""
            return _DOWNLOAD_INSTRUCTIONS.format(
                **asdict(self), local_storage=local_storage
            )
        else:
            data = asdict(self)
            if self._trial_mode:
                data["addendum"] = _TRIAL_MODE_ADDENDUM
            else:
                data["addendum"] = ""
            return _STORAGE_INSTRUCTIONS.format(**data)


_UNKNOWN_JOB_ID = "Unknown, see docs to learn how to set this"


def _log_result(
    job_secs_since_epoch: int,
    job_id: Optional[str],
    download_key: Optional[str],
    decryption_key: Optional[str],
    report_path: Optional[Path],
    trial_mode: bool,
):
    """Log a ``ReportResult``."""
    if job_id is None:
        job_id = _UNKNOWN_JOB_ID
    job_time = datetime.fromtimestamp(job_secs_since_epoch, timezone.utc).isoformat()
    report = ReportResult(
        job_time=job_time,
        job_id=job_id,
        download_key=download_key,
        decryption_key=decryption_key,
        report_path=report_path,
        _trial_mode=trial_mode,
    )
    _LOGGER.warning(report)


def _load_module(name: str):
    """Load a Python module from the pre-loaded Sciagraph shared library."""
    # The symbols in the current running process:
    this = PyDLL(None)
    try:
        f = getattr(this, f"PyInit_{name}")
    except AttributeError:
        # We're not running under Sciagraph:
        assert getattr(this, "sciagraph_initialize", None) is None
        return None
    f.restype = py_object
    module = f()
    sys.modules[name] = module
    return module


_sciagraph_jobs = _load_module("_sciagraph_jobs")


def set_job_id(job_id: str):
    """
    Set the current job's ID; it will then be included in the resulting report
    and logged message.
    """
    if _sciagraph_jobs:
        _sciagraph_jobs.set_job_id(job_id)


def _start_job(job_id: str, output_path: Optional[Union[PathLike, str]] = None):
    """
    Start a new job.
    """
    if output_path is not None:
        output_path = str(output_path)
    if _sciagraph_jobs:
        if output_path is not None:
            makedirs(output_path, exist_ok=True)
        _sciagraph_jobs.start_job(job_id, output_path)


def _finish_job():
    """
    Finish the current job.
    """
    if _sciagraph_jobs:
        _sciagraph_jobs.finish_job()


@contextmanager
def profile_job(job_id: str, output_path: Union[PathLike, str]):
    """
    A context manager that starts a job on entry and finishes it on exit.
    """
    _start_job(job_id, output_path)
    try:
        yield
    finally:
        _finish_job()
